/* 
	Boundary.java

	Title:			Kontsevich graph homology calculator
                                PhD thesis, Cornell University, Aug 2002
	Author:			Ferenc Gerlits
	Description:	constructs the matrix for the boundary map
*/

package Konts;

import java.util.Vector;
import java.io.*;

public class Boundary
{
  public static void main(String argv[]) throws IOException {
    int gix, i, j, ii, p, graphNo, edgeNo;
    Graph oldGraph = null, newGraph;
    Vector column, graphBag;
    int[] mxelt = new int[2], oldelt = new int[2];
    boolean found;
    Permutation sigma;
    
    if (argv.length < 3) {
      System.out.println("Usage: Boundary <original graphs> "+
                         "<contracted graphs> <boundary matrix>.");
      System.exit(1);
      }

    FileInputStream infs = new FileInputStream(argv[0]);             
    ObjectInputStream ins = new ObjectInputStream(infs);
    FileInputStream outfs = new FileInputStream(argv[1]);
    ObjectInputStream outs = new ObjectInputStream(outfs);
    FileOutputStream matrixfs = new FileOutputStream(argv[2], false);
    PrintWriter matrixs = new PrintWriter(matrixfs);
    
    graphBag = new Vector(100,100);
    try {
        while (true) {
          oldGraph = (Graph)outs.readObject();
          graphBag.addElement(oldGraph);
          }
        }
      catch(EOFException e) { }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
    outfs.close();
    
    column = new Vector(100,100);
      
    graphNo = 0;
    while(true) { 
      try {
        oldGraph = (Graph)ins.readObject();
        }
      catch (EOFException e) {
        break;
        }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
      
      System.out.print(graphNo +" ");
      column.removeAllElements();
      edgeNo=0;
            
      for (i=0; i<oldGraph.size; i++)
        for (j=i+1; j<oldGraph.size; j++) {

          edgeNo += oldGraph.m[i][j];
          if (oldGraph.m[i][j] != 1)
            continue;

          newGraph = new Graph(oldGraph);
          newGraph.contractEdge(i, j);
          if (newGraph.toNormalForm(2) != null) {
            found = false;
            for (ii=0; ii<graphBag.size(); ii++)
              if (newGraph.equals((Graph)graphBag.elementAt(ii))) {
              found = true;
              break;
              }
            if (!found) {
              System.out.println("Error: can't find graph.");
              System.exit(1);
              }
            mxelt[0] = newGraph.coeff;	// contracted graph is mxelt[0]
            mxelt[1] = ii;				//  times graph number ii in bag
          
            if (j % 2 == 1)
              mxelt[0] = -mxelt[0];	// multiply coeff by -1^larger index
              
            found = false;
            for (p=0; p<column.size(); p++) {
              oldelt = (int[])column.elementAt(p);
              if (ii == oldelt[1]) {
                if ((oldelt[0] += mxelt[0]) == 0)	// increase coefficient
                  column.removeElementAt(p);		// if graph occurred in
                found = true;						// this column already...
                break;
                }
              }
            if (!found) 							// ... add new element
              column.addElement(mxelt.clone());		// to column if not.
            }
          }
          
      for (p=0; p<column.size(); p++) {
        mxelt = (int[])column.elementAt(p);
        matrixs.println((graphNo+1) +"\t"+ (mxelt[1]+1) +"\t"+ mxelt[0]);
        }
      if (column.size() == 0)
        matrixs.println((graphNo+1) +"\t"+ 1 +"\t"+ 0);
      matrixs.println();
      graphNo++;
      }		// end of while(true) loop: process next graph
            
    ins.close();   infs.close();
    matrixs.flush();  matrixs.close();  matrixfs.close();

    System.out.println("done. "+ new java.util.Date());
    }
}
