/* 
	Contract.java

	Title:			Kontsevich graph homology calculator
                                PhD thesis, Cornell University, Aug 2002
	Author:			Ferenc Gerlits
	Description:	given the trivalent graphs, construct all the
                        at-least-trivalent graphs.
  
    syntax: java Konts.Contract graph4_6.out graph4_5.out
      makes the rank 4 graphs on 5 vertices out of the rank 4 graphs on 
      6 vertices.  After contraction, graph is transformed to normal form, 
      and its coefficient is reset to 1.
*/

package Konts;

import java.util.Vector;
import java.io.*;

public class Contract
{
  public static void main (String args[]) throws IOException {
    int i, j, ii, jj, k, m, n;
    int dist[];
    boolean isNew, found;
    Graph oldGraph, newGraph;
    Vector oldBag, newBag;
    
    if (args.length != 2) {
      System.out.println("Usage: Contract inputgraphs outputgraphs.");
      System.exit(1);
      }
    
    oldBag = new Vector(10,10);
    
    FileInputStream infs = null;
    ObjectInputStream ins;
    infs = new FileInputStream(args[0]);
    ins = new ObjectInputStream(infs);
    try {
      while (true) {
        oldGraph = (Graph)ins.readObject();
        oldBag.addElement(oldGraph);
        }
      }
    catch(EOFException e) { }
    catch (ClassNotFoundException e) {
      System.out.println("Class Graph not found.");
      System.exit(1);
      }
    infs.close();
    System.out.println(oldBag.size() + " graphs read.");
    System.out.print("Processing graphs:");

    newBag = new Vector(100,100);

    for (n=0; n<oldBag.size(); n++) {
      oldGraph = (Graph)oldBag.elementAt(n);
      System.out.print(" " + n);


      for (i=0; i<oldGraph.size; i++)
        for (j=i+1; j<oldGraph.size; j++) {

          if (oldGraph.m[i][j] != 1)
            continue;

          newGraph = new Graph(oldGraph);
          newGraph.contractEdge(i, j);
          newGraph.toNormalForm(1);

          found = false;
          for (ii=0; ii<newBag.size(); ii++)
            if (newGraph.equals((Graph)newBag.elementAt(ii))) {
            found = true;
            break;
            }
          if (!found) {
            System.out.print("*");
            newBag.addElement(newGraph);
            }
          }
      
      }
    
    FileOutputStream outfs = new FileOutputStream(args[1]);
    ObjectOutputStream outs = new ObjectOutputStream(outfs);
    for (n=0; n<newBag.size(); n++)
      outs.writeObject((Graph)newBag.elementAt(n));
    outs.close();
    outfs.close();

    System.out.println();
    System.out.println("Written " + n + " graphs at " + new java.util.Date());
    }
} 
