/* 
	ListGraphs.java

	Title:			Kontsevich graph homology calculator
                                PhD thesis, Cornell University, Aug 2002
	Author:			Ferenc Gerlits
	Description:	listing equivalence classes of graphs
*/

package Konts;

import java.util.Vector;
import java.io.*;

public class ListGraphs 
{
  static public void main(String[] args) throws IOException
	{
    int i, j, k, n;
    boolean isNew;
    Graph oldGraph, newGraph;
    Vector oldBag, newBag;
    
    if (args.length != 2) {
      System.out.println("Usage: ListTrivalent inputgraphs "+
                               "outputgraphs");
      System.exit(1);
      }
    
    oldBag = new Vector(10,10);
    
    if (args[0].equals("null")) {	// generating the rank 2 graphs
      Graph basicGraph = new Graph(2);
      basicGraph.m[0][1] = 3;
      basicGraph.noEdges = 1;
      basicGraph.allEdges = 3;
      basicGraph.noMultiples = 2;
      oldBag.addElement(basicGraph);

      basicGraph = new Graph(2);
      basicGraph.m[0][0] = basicGraph.m[0][1] = basicGraph.m[1][1] = 1;
      basicGraph.noEdges = 1;
      basicGraph.allEdges = 3;
      basicGraph.noLoops = 2;
      oldBag.addElement(basicGraph);
      }
    else {
      FileInputStream fs = null;
      ObjectInputStream s;
      fs = new FileInputStream(args[0]);
      s = new ObjectInputStream(fs);
      try {
        while (true) {
          oldGraph = (Graph)s.readObject();
          oldBag.addElement(oldGraph);
          }
        }
      catch(EOFException e) { }
      catch (ClassNotFoundException e) {
        System.out.println("Class not found.");
        System.exit(1);
        }
      fs.close();
      }
    System.out.print(oldBag.size() + " graphs read.\r\n"+
                      "Processing graphs:");
  
    newBag = new Vector(100,100);
   
    for (n=0; n<oldBag.size(); n++) {
      oldGraph = (Graph)oldBag.elementAt(n);
      System.out.print(" " + n);
    
      for (i=0; i<oldGraph.allEdges; i++)
        for (j=i; j<oldGraph.allEdges; j++) {	// counting edges with
          newGraph = oldGraph.addEdge(i, j);	// multiplicity
          newGraph.toNormalForm(1);

          isNew = true;      
          for (k=0; k<newBag.size(); k++)
            if (newGraph.equals((Graph)newBag.elementAt(k))) {
              isNew = false;
              System.out.print(".");
              break;
              }						
          if (isNew) {			// not in the bag yet 
            newBag.addElement(newGraph);
            System.out.print("*");
            }
          }
          
      for (i=0; i<oldGraph.noEdges; i++) {
        newGraph = oldGraph.addALoop(i);	// the i'th edge; counting
        newGraph.toNormalForm(1);
          
        isNew = true;
        for (k=0; k<newBag.size(); k++)
          if (newGraph.equals((Graph)newBag.elementAt(k))) {
            isNew = false;
            System.out.print(".");
            break;
            }						
        if (isNew) {			// not in the bag yet 
          newBag.addElement(newGraph);
          System.out.print("*");
          }
        }
            
      }	// end of "for n" loop

    FileOutputStream fs = new FileOutputStream(args[1]);
    ObjectOutputStream s = new ObjectOutputStream(fs);
    for (n=0; n<newBag.size(); n++)
      s.writeObject((Graph)newBag.elementAt(n));
    s.close();
    fs.close();

    System.out.println("\r\nWritten " + n + " graphs at " +
                       new java.util.Date());
    }
}
